#!/usr/bin/perl -w

##  PROJECT HONEY POT MOVABLE TYPE PLUGIN COMPONENT
##  For more information visit: http://www.projecthoneypot.org/
##  Copyright (C) 2004, Unspam, LLC
##  
##  This program is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation; either version 2 of the License, or
##  (at your option) any later version.
##  
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##  
##  You should have received a copy of the GNU General Public License
##  along with this program; if not, write to the Free Software
##  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
##  02111-1307  USA

package MT::Plugins::ProjectHoneyPot;

use vars qw($VERSION);
$VERSION = "0.1";

use strict;
use warnings;
use MT::Template::Context;

my($MT_DIR);
BEGIN {
	if ($0 =~ m!(.*[/\\])!) {
		$MT_DIR = $1;
	} else {
		$MT_DIR = './';
	}
}

my $plugin_vars = &get_data;

eval{ require MT::Plugin };
unless ($@) {
	my $phpot_plugin = { name => "Project Honey Pot MT Plugin v.$VERSION", description => 'Installs spambot-trapping honey pots on your Movable Type-based site.', config_link => '../mt-projecthoneypot.cgi', doc_link => 'http://www.projecthoneypot.org/mt_documentation.php' };
	MT->add_plugin( new MT::Plugin($phpot_plugin) );
}

if ($plugin_vars) {
	if ($plugin_vars->{'honeypot_tag'}) {
		MT::Template::Context->add_tag(HoneyPotLink => sub { &do_return_link });
	}
	if ($plugin_vars->{'global_filter'}) {
		MT::Template::Context->add_global_filter(honeypotlink => sub { &do_add_link });
	}
	
	if ($plugin_vars->{'text_filter'}) {
		my $text_filter_label = "Honey Pot Link";
		my $prefilter = $plugin_vars->{'prefilter'};
		my $prefilter_label = $plugin_vars->{'prefilter_label'};
		if ($prefilter && $prefilter_label) {
			$text_filter_label .= " (+".$prefilter_label.")";
		}
		MT->add_text_filter('phpot_textfilter' => { label => $text_filter_label, docs => 'http://www.projecthoneypot.org/mt_documentation.php', on_format => \&process_text });
	}
}

sub get_data {
	use MT::PluginData;
	use MT::App::CMS;
	
	my ($pdata) = MT::PluginData->load({ plugin => 'mt-projecthoneypot', key => 'phpot_plugin' });
	if ($pdata) {
		return ($pdata->data);
	}
	return 0;
}

sub get_url {
	my $app = MT::App::CMS->new;

	my $hpot_script = $plugin_vars->{'script_location'};
	my $script_dir = $app->path;
	my $server_domain = $ENV{SERVER_NAME};
	if ($hpot_script) {
		return "http://$server_domain$script_dir$hpot_script";
	}
	return 0;
}

sub get_link_format {
	my $param = $_[0];
	my $pattern = "";
	if (open (PATTERNS, "$MT_DIR/plugins/projecthoneypot/link_formats.conf")) {
		my @patterns = ();
		my @pattern_formats = split(/:/,$plugin_vars->{'link_patterns'});
		my $pattern_count = 0;
		while ($pattern = <PATTERNS>) {
			chomp($pattern);
			$pattern =~ s/^\[\![a-zA-Z0-9\:\;\.\,\-\_\+\= ]*\!\][ ]*//g;
			if ($pattern) {
				if (!$param) {
					if ($pattern_formats[$pattern_count]) {
						push (@patterns, $pattern);
					}
				} else {
					push (@patterns, $pattern);
				}
				$pattern_count++;
			}
		}
		close(PATTERNS);
		if (!$param) {
			$pattern = $patterns[int(rand(@patterns))];
		} else {
			my @params = split(/,/,$param);
			my $pattern_num = $params[int rand scalar @params];
			$pattern = $patterns[((int $pattern_num)-1)];
		}
	}
	if (!$pattern) { $pattern = "<a href='[#url#]' style='display: none;'>[#random#]</a>"; }
	return $pattern;
}

sub do_return_link {
	my($ctx, $args) = @_;
	my $param = $args->{'style'};
	my @params = (0,0);
	if ($param && ($param ne "default")) {
		@params = split(/:/,$param);
	}
	my $url = &get_url;
	if ($url) {
		my $link_format = &get_link_format($params[0]);
		if ($link_format) {
			$link_format =~ s/\[\#url\#\]/$url/g;
			$link_format = &replace_random($link_format);
			return $link_format;
		}
	}
	return 0;
}

sub replace_random {
	my $link_format = $_[0];
	my @seed=qw(a b c d e f g h i j k l m n o p q r s t u v w x y z a b c d e f g h i j k l m n o p r s t u v w y);

	my $length=int(rand(4))+4;
	my $random_replace = "";
	foreach(1..$length){
		$random_replace.=$seed[rand(@seed)];
	}
	while ($link_format =~ s/\[\#random\#\]/$random_replace/) {
		$length=int(rand(5))+4;
		$random_replace = "";
		foreach(1..$length){
			$random_replace.=$seed[rand(@seed)];
		}
	}
	
	return $link_format;
}

sub do_add_link {
	my ($str, $param, $ctx) = @_;
	my @params = (0,0);
	if ($param && ($param ne "default")) {
		@params = split(/:/,$param);
	}
	if ($params[1] ne "top" && $params[1] ne "bottom" && $params[1] ne "random") {
		$params[1] = $plugin_vars->{'linkinsert_default'};
		if (!$params[1]) { $params[1] = "bottom"; }
	}
	my $url = &get_url;
	if ($url) {
		if (!($str =~ /$url/)) {
			my $link_format = &get_link_format($params[0]);
			if ($link_format) {
				$link_format =~ s/\[\#url\#\]/$url/g;
				$link_format = &replace_random($link_format);
				my $mod_str = "";
				if ($params[1] eq "top") {
					$mod_str = $link_format.$str;
				} elsif ($params[1] eq "random") {
					$mod_str = &do_random_insert($str,$link_format);
				} elsif ($params[1] eq "bottom") {
					$mod_str = $str.$link_format;
				}
				return $mod_str;
			}
		}
	}
	return $str;
}

sub process_text {
	my ($str, $ctx) = @_;
	my $filters = MT->all_text_filters;
	if (exists($filters->{$plugin_vars->{'prefilter'}})) {
		my @tf = ($plugin_vars->{'prefilter'});
		$str = MT->apply_text_filters($str, \@tf);
	}
	my $url = &get_url;
	if ($url) {
		if (!($str =~ /$url/)) {
			my $link_format = &get_link_format(0);
			if ($link_format) {
				$link_format =~ s/\[\#url\#\]/$url/g;
				my $mod_str = "";
				if ($plugin_vars->{'textfilter_default'} eq "top") {
					$mod_str = $link_format.$str;
				} elsif ($plugin_vars->{'textfilter_default'} eq "random") {
					$mod_str = &do_random_insert($str,$link_format);
				} elsif ($plugin_vars->{'textfilter_default'} eq "bottom") {
					$mod_str = $str.$link_format;
				}
				return $mod_str;
			}
		}
	}
	return $str;
}

sub do_random_insert {
	my ($str,$lnk) = @_;
	my $rand_pos = int(rand(length $str));
	my $cur_char = "";
	my $mod_str = "";
	my $in_tag = 0;
	my $in_link = 0;
	my $inserted = 0;
	for (my $i = 0; $i < (length $str); $i++) {
		$cur_char = substr($str, $i, 1);
		if ($cur_char eq "<") {
			$in_tag=1;
			if (substr($str,$i+1,1) eq "a") {
				$in_link++;
			} elsif (substr($str,$i+1,2) eq "/a") {
				$in_link--;
			}
		} elsif ($cur_char eq ">" && $in_tag>0) {
			$in_tag=0;
		}
		$mod_str .= $cur_char;
		if (!($cur_char =~ /[a-zA-Z0-9\&\#\;]/)) {
			if (!$in_tag && !$in_link && $i>$rand_pos && !$inserted) {
				$mod_str .= $lnk;
				$inserted = 1;
			}
		}
	}
	return $mod_str;
}

1;
